﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.Text;

namespace PTCtpClientSample
{
    //Current Contract values
    /*
    public enum EApsWebServicesResponseCodes
    {
        Success = 0,
        SuccessWihoutValidation = 1,
        InvalidDateFormat = 2,
        InvalidDatePast = 3,
        InvalidDateFuture = 4,
        InvalidUserCredentials = 5,
        InvlaidUserPermissions = 6,
        NoServiceListening = 7,
        NoInstanceWithThatName = 8,
        ValidationTimeout = 9,
        NoServerManager = 10,
        InvalidScenarioName = 11,
        FailedToBroadcast = 12,
        ProcessingTimeout = 13,
        Failure = 14,
    }

    [ServiceContract]
    public interface IExtraServicesWebService
    {
        [OperationContract]
        EApsWebServicesResponseCodes AdvanceClock(string a_user, string a_password, string a_dateTime);

        [OperationContract]
        EApsWebServicesResponseCodes AdvanceClockTicks(string a_user, string a_password, long a_dateTimeTicks);

        [OperationContract]
        EApsWebServicesResponseCodes PublishLive(string a_user, string a_password, int a_publishType);

        [OperationContract]
        EApsWebServicesResponseCodes PublishByName(string a_user, string a_password, int a_publishType, string a_scenarioName);

        [OperationContract]
        CtpResponse CTP(string a_userName, string a_password, CtpRequest a_ctpRequest);
    }
     */

    class Program
    {
        static void Main(string[] args)
        {
            if (args == null || args.Length != 5)
            {
                Console.WriteLine("Need 5 parameters separated by spaces: URL username password ItemExternalId WarehouseExternalId");
                Environment.Exit(1);
            }
            string url = args[0];
            string username = args[1];
            string pwd = args[2];
            string item = args[3];
            string warehouse = args[4];
            if (string.IsNullOrEmpty(item))
            {
                item = "Item 1";
            }
            if (string.IsNullOrEmpty(warehouse))
            {
                warehouse = "Warehouse 1";
            }

            //Soap client

            //Setup the binding. This is done in code so the config file is not needed
            BasicHttpBinding binding = new BasicHttpBinding(BasicHttpSecurityMode.None);
            binding.BypassProxyOnLocal = true;
            binding.CloseTimeout = new TimeSpan(0, 2, 0);
            binding.OpenTimeout = new TimeSpan(0, 0, 30);
            binding.SendTimeout = new TimeSpan(0, 10, 0);
            binding.AllowCookies = false;
            binding.HostNameComparisonMode = HostNameComparisonMode.StrongWildcard;
            binding.MaxBufferPoolSize = 524288;
            binding.MaxBufferSize = int.MaxValue;
            binding.MaxReceivedMessageSize = int.MaxValue;
            binding.TextEncoding = Encoding.UTF8;
            binding.TransferMode = TransferMode.Buffered;
            binding.MessageEncoding = WSMessageEncoding.Text;

            APSWebServiceRef.ExtraServicesWebServiceClient client = new APSWebServiceRef.ExtraServicesWebServiceClient(binding, new EndpointAddress(url));

            //Create a typed request
            APSWebServiceRef.CtpRequest ctpRequest = new APSWebServiceRef.CtpRequest();

            //Example CTP request
            List<APSWebServiceRef.CtpRequestLine> ctpRequestLines = new List<APSWebServiceRef.CtpRequestLine>();
            ctpRequestLines.Add(new APSWebServiceRef.CtpRequestLine()
            {
                LineId = 1,
                ItemExternalId = item,
                WarehouseExternalId = warehouse,
                RequiredQty = 100,
                NeedDate = DateTime.Now.Add(TimeSpan.FromDays(7))
            });

            //Connection settings
            ctpRequest.TimeoutMinutes = 3;
            ctpRequest.CtpRequestLines = ctpRequestLines.ToArray();
            ctpRequest.ReserveMaterialsAndCapacity = true;
            ctpRequest.SchedulingType = APSWebServiceRef.CtpDefsESchedulingType.Optimize;
            Write(ctpRequest);
            DateTime start = DateTime.Now;
            Console.WriteLine($"Request sent at '{start}'");

            //Sending a request requires the username and password of an APS user with appropriate permissions, generally an admin.
            var resp = client.CTP(username, pwd, ctpRequest);
            Console.WriteLine($"Response received after '{DateTime.Now.Subtract(start)}'");
            Write(resp);
            Console.WriteLine("Press any key to exit.");
            Console.ReadLine();
        }

        private static void Write(APSWebServiceRef.CtpRequest a_req)
        {
            Console.WriteLine($"    Priority: '{a_req.Priority}'");
            Console.WriteLine($"    ReserveMaterialsAndCapacity: '{a_req.ReserveMaterialsAndCapacity}'");
            Console.WriteLine($"    SchedulingType: '{a_req.SchedulingType}'");
            Console.WriteLine($"    TimeoutMinutes: '{a_req.TimeoutMinutes}'");
            Console.WriteLine($"    CtpRequestLines.Length: '{a_req.CtpRequestLines.Length}'");
            foreach (var reqLine in a_req.CtpRequestLines)
            {
                Console.WriteLine($"        LineId: '{reqLine.LineId}'");
                Console.WriteLine($"        WarehouseExternalId: '{reqLine.WarehouseExternalId}'");
                Console.WriteLine($"        ItemExternalId: '{reqLine.ItemExternalId}'");
                Console.WriteLine($"        NeedDate: '{reqLine.NeedDate}'");
                Console.WriteLine($"        RequiredQty: '{reqLine.RequiredQty}'");
            }
        }

        private static void Write(APSWebServiceRef.CtpResponse a_resp)
        {
            Console.WriteLine($"    Code: '{a_resp.ReturnCode}'");
            Console.WriteLine($"    ErrorMessage: '{a_resp.ErrorMessage}'");
            if (a_resp.CtpResponseLines == null)
            {
                Console.WriteLine($"    CtpResponseLines: null");
            }
            else
            {
                Console.WriteLine($"    CtpResponseLines.Length: {a_resp.CtpResponseLines.Length}");
                foreach (var respLine in a_resp.CtpResponseLines)
                {
                    Console.WriteLine($"        LineId: '{respLine.LineId}'");
                    Console.WriteLine($"        ScheduledStartDate: '{respLine.ScheduledStartDate}'");
                    Console.WriteLine($"        ScheduledStartDate: '{respLine.ScheduledEndDate}'");
                }
            }
        }
    }
}
